@@ -4,7 +4,7 @@
 #'
 #' @param id,input,output,session Internal parameters for {shiny}.
 #'
-#' @importFrom shinyjs inlineCSS
+#' @importFrom shinyjs inlineCSS useShinyjs
 #' 
 #' @noRd 
 #'
@@ -23,15 +23,22 @@ mod_genes_view_ui <- function(id){
           ),
           column(width = 12,
                  div(style = "position:absolute;right:1em;", 
-                     actionButton(ns("exit"), "Exit",icon("times-circle"), class = "btn btn-danger"),  br(), br(),
-                     actionButton(ns("goMap"), "Next",icon("arrow-circle-right"), class = "btn btn-success")
+                     div(
+                       actionButton(ns("goQTL"), "Go to QTL",icon("arrow-circle-left", verify_fa = FALSE), class = "btn btn-primary"),
+                       actionButton(ns("goMap"), label = div("Go to Map", icon("arrow-circle-right", verify_fa = FALSE)), class = "btn btn-primary"))
                  )
           ),
           tags$h2(tags$b("VIEWgenome")), br(), hr(),
           
           column(6,
+                 column(12,
+                        box(
+                          background = "light-blue",
+                          "Required inputs (*)", br(),
+                        )
+                 ),
                  column(6,
-                        box(width = 12, solidHeader = TRUE, status="info", title = h4("Select phenotypes"),
+                        box(width = 12, solidHeader = TRUE, status="info", title = "Select phenotypes *",
                             pickerInput(ns("phenotypes"),
                                         label = h4("Phenotypes:"),
                                         choices = "This will be updated",
@@ -45,7 +52,7 @@ mod_genes_view_ui <- function(id){
                         ), br(),
                  ),
                  column(6,
-                        box(width = 12, solidHeader = TRUE, status="info", title = h4("Select linkage group"),
+                        box(width = 12, solidHeader = TRUE, status="info", title = "Select linkage group *",
                             selectInput(inputId = ns("group"), label = p("Linkage group:"), choices = 1:15, selected = 1),
                         ), br(),
                  )
@@ -56,34 +63,72 @@ mod_genes_view_ui <- function(id){
                       value = c(0, 20), step = 1), 
           uiOutput(ns("interval"))
         ),
-        box(width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = TRUE, status="primary", title = h4("QTL profile"),
-            column(2,
-                   downloadBttn(ns('bn_download'), style = "gradient", color = "royal")
+        box(id = ns("box_profile"),width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = TRUE, status="primary", title = actionLink(inputId = ns("profileID"), label = "QTL profile"),
+            column(12,
+                   box(
+                     width = 5, background = "light-blue",
+                     "* QTL analysis files or viewpoly object or example dataset (check `Input data` tab)"
+                   )
+            ), 
+            column(3,
+                   useShinyjs(),
+                   tags$head(tags$style(".butt{background-color:#add8e6; border-color: #add8e6; color: #337ab7;}")),
+                   downloadButton(ns('bn_download'), "Download", class = "butt")
             ),
-            column(10,
+            column(3,
                    radioButtons(ns("fformat"), "File type", choices=c("png","tiff","jpeg","pdf"), selected = "png", inline = T)
+            ),                     
+            column(2,
+                   numericInput(ns("width_profile"), "Width (mm)", value = 180),
+            ),
+            column(2,
+                   numericInput(ns("height_profile"), "Height (mm)", value = 120),
+            ),
+            column(2,
+                   numericInput(ns("dpi_profile"), "DPI", value = 300)
             ), br(),
             column(12,
                    hr(),
                    plotlyOutput(ns("plot_qtl"))
             )
         ), br(),
-        box(width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = h4("Linkage Map position (cM) x Physical position (Mp)"),
-            column(2,
-                   downloadBttn(ns('bn_download_phi'), style = "gradient", color = "royal")
+        box(id = ns("box_phi"),width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = actionLink(inputId = ns("phiID"), label = "Linkage Map position (cM) x Physical position (Mb)"),
+            column(12,
+                   box(
+                     width = 5, background = "light-blue",
+                     "* MAPpoly linkage map files or viewpoly object or example dataset (check `Input data` tab)", 
+                   )
+            ), 
+            column(3,
+                   downloadButton(ns('bn_download_phi'), "Download", class = "butt")
             ),
-            column(10,
+            column(3,
                    radioButtons(ns("fformat_phi"), "File type", choices=c("png","tiff","jpeg","pdf"), selected = "png", inline = T)
+            ),                     
+            column(2,
+                   numericInput(ns("width_phi"), "Width (mm)", value = 180),
+            ),
+            column(2,
+                   numericInput(ns("height_phi"), "Height (mm)", value = 120),
+            ),
+            column(2,
+                   numericInput(ns("dpi_phi"), "DPI", value = 300)
             ), br(),
             column(12,
                    hr(),
                    plotlyOutput(ns("plot_pos"))
             )
         ), br(),
-        box(width = 12, height = 1000, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = h4("JBrowseR"),
+        box(id = ns("box_jbrowser"), width = 12, height = 1000, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = actionLink(inputId = ns("jbrowserID"), label = "JBrowseR"),
+            column(12,
+                   box(
+                     width = 5, background = "light-blue",
+                     "* Reference genome FASTA (check `Input data` tab)"
+                   )
+            ), 
             column(12,
                    column(6,
-                          actionButton(ns("create_server"), "Open JBrowseR",icon("power-off"))
+                          actionButton(ns("create_server"), "Open JBrowseR",icon("power-off", verify_fa = FALSE))
                    ),
                    column(6,
                           div(style = "position:absolute;right:1em;", 
@@ -95,8 +140,17 @@ mod_genes_view_ui <- function(id){
             column(12, br(), hr(),
                    JBrowseROutput(ns("browserOutput"))
             ), br()),
-        box(width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = h4("Annotation table"),
-            DT::dataTableOutput(ns("genes_ano"))
+        box(id = ns("box_anno"),width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = actionLink(inputId = ns("annoID"), label = "Annotation table"),
+            column(12,
+                   box(
+                     width = 5, background = "light-blue",
+                     "* Reference genome FASTA (check `Input data` tab)", br(),
+                     "* Genome annotation GFF (check `Input data` tab)"
+                   )
+            ), 
+            column(12,
+                   DT::dataTableOutput(ns("genes_ano"))
+            )
         )
       )
     )
@@ -107,7 +161,7 @@ mod_genes_view_ui <- function(id){
 #'
 #' @importFrom JBrowseR serve_data renderJBrowseR assembly track_feature tracks default_session JBrowseR JBrowseROutput 
 #' @importFrom plotly event_data layout
-#' @importFrom shinyjs inlineCSS
+#' @importFrom shinyjs inlineCSS js
 #' @importFrom dplyr `%>%`
 #'
 #' @noRd 
@@ -121,8 +175,21 @@ mod_genes_view_server <- function(input, output, session,
   pheno <- LG <- l.dist <- g.dist <- high <- mk.names <- track_variant <- track_alignments <- track_wiggle <- NULL
   start <- end <- seqid <- NULL
   
-  observeEvent(input$exit, {
-    stopApp()
+  #Collapse boxes
+  observeEvent(input$profileID, {
+    js$collapse(ns("box_profile"))
+  })
+  
+  observeEvent(input$phiID, {
+    js$collapse(ns("box_phi"))
+  })
+  
+  observeEvent(input$jbrowserID, {
+    js$collapse(ns("box_jbrowser"))
+  })
+  
+  observeEvent(input$annoID, {
+    js$collapse(ns("box_anno"))
   })
   
   observe({
@@ -165,6 +232,11 @@ mod_genes_view_server <- function(input, output, session,
                       selected = "map")
   })
   
+  observeEvent(input$goQTL, {
+    updateTabsetPanel(session = parent_session, inputId = "viewpoly",
+                      selected = "qtl")
+  })
+  
   # Plot QTL bar
   qtl.int <- reactive({
     if(!is.null(loadQTL())){
@@ -243,41 +315,34 @@ mod_genes_view_server <- function(input, output, session,
   
   # Plot QTL profile
   output$plot_qtl <- renderPlotly({
-    if(!is.null(loadQTL())){
-      idx <- which(unique(loadQTL()$profile$pheno) %in% input$phenotypes)
-      pl <- plot_profile(profile = loadQTL()$profile, qtl_info = loadQTL()$qtl_info, selected_mks = loadQTL()$selected_mks,
-                         pheno.col = idx,
-                         lgs.id = as.numeric(input$group),
-                         range.min = input$range[1],
-                         range.max = input$range[2], 
-                         by_range=T, 
-                         software = loadQTL()$software)
-      ggplotly(source = "qtl_profile", pl, tooltip=c("Trait","Position (cM)")) %>% layout(legend = list(orientation = 'h', y = -0.3))
-    } else 
-      stop("Upload the QTL information in upload session to access this feature.")
+    validate(
+      need(!is.null(loadQTL()), "Upload the QTL information in upload session to access this feature.")
+    )
+    idx <- which(unique(loadQTL()$profile$pheno) %in% input$phenotypes)
+    pl <- plot_profile(profile = loadQTL()$profile, qtl_info = loadQTL()$qtl_info, selected_mks = loadQTL()$selected_mks,
+                       pheno.col = idx,
+                       lgs.id = as.numeric(input$group),
+                       range.min = input$range[1],
+                       range.max = input$range[2], 
+                       by_range=T, 
+                       software = loadQTL()$software)
+    ggplotly(source = "qtl_profile", pl, tooltip=c("Trait","Position (cM)")) %>% 
+      layout(legend = list(orientation = 'h', y = -0.3), 
+             modebar = list(
+               remove = c("toImage", 
+                          "hovercompare", 
+                          "hoverCompareCartesian")),
+             clickmode ="none",
+             dragmode = FALSE)
   })
   
   # cM x Mb
   output$plot_pos <- renderPlotly({
-    if(!is.null(loadMap()))
-      if(loadMap()$software == "polymapR") stop("Feature not implemented for software polymapR.") 
-    
-    if(is.null(loadMap()$ph.p1)) stop("Upload map information in the upload session to access this feature.")
-    
-    map.lg <- loadMap()$maps[[as.numeric(input$group)]]
-    
-    map.lg$high <- map.lg$g.dist
-    map.lg$high[round(map.lg$l.dist,5) < input$range[1] | round(map.lg$l.dist,5) > input$range[2]] <- "black"
-    map.lg$high[round(map.lg$l.dist,5) >= input$range[1] & round(map.lg$l.dist,5) <= input$range[2]] <- "red"
-    
-    map.lg$high <- as.factor(map.lg$high)
-    p <- ggplot(map.lg, aes(x=l.dist, y = g.dist/1000, colour = high, text = paste("Marker:", mk.names, "\n", 
-                                                                                   "Genetic:", round(l.dist,2), "cM \n",
-                                                                                   "Genomic:", g.dist/1000, "Mb"))) +
-      geom_point() + scale_color_manual(values=c('black','red')) + 
-      theme(legend.position = "none") + 
-      labs(x = "Linkage map (cM)", y = "Reference genome (Mb)") +
-      theme_bw()
+    validate(
+      need(!(!is.null(loadMap()) & loadMap()$software == "polymapR"), "Feature not implemented for software polymapR."),
+      need(!is.null(loadMap()$ph.p1), "Upload map information in the upload session to access this feature.")
+    )
+    p <- plot_cm_mb(loadMap(), input$group, input$range[1], input$range[2])
     
     max_updated = reactive({
       dist <- loadMap()$maps[[as.numeric(input$group)]]$l.dist
@@ -289,7 +354,13 @@ mod_genes_view_server <- function(input, output, session,
       updateSliderInput(inputId = "range", max = round(max_updated(),2))
     })
     
-    ggplotly(p, tooltip="text") %>% layout(showlegend = FALSE)
+    ggplotly(p, tooltip="text") %>% layout(showlegend = FALSE, 
+                                           modebar = list(
+                                             remove = c("toImage", 
+                                                        "hovercompare", 
+                                                        "hoverCompareCartesian")),
+                                           clickmode ="none",
+                                           dragmode = FALSE)
   })
   
   # Open JBrowser server 
@@ -307,9 +378,9 @@ mod_genes_view_server <- function(input, output, session,
         if(grepl("^http", loadJBrowse_gff3())){
           gff.dir <- tempfile()
           download.file(loadJBrowse_gff3(), destfile = gff.dir)
-          gff <- vroom(gff.dir, delim = "\t", skip = 3, col_names = F)
+          gff <- vroom(gff.dir, delim = "\t", skip = 3, col_names = F, progress = FALSE, show_col_types = FALSE)
         } else {
-          gff <- vroom(loadJBrowse_gff3(), delim = "\t", skip = 3, col_names = F)
+          gff <- vroom(loadJBrowse_gff3(), delim = "\t", skip = 3, col_names = F, progress = FALSE, show_col_types = FALSE)
         }
       } else path.gff <- gff <- NULL
     } else path.gff <- gff <- NULL
@@ -332,18 +403,32 @@ mod_genes_view_server <- function(input, output, session,
       } else path.wig <- NULL
     } else path.wig <- NULL
     
-    if(is.null(loadJBrowse_fasta()) & !is.null(loadExample())){
-        path.fa <- loadExample()$fasta
-        path.gff <- loadExample()$gff3
-        gff.dir <- tempfile()
-        download.file(loadExample()$gff3, destfile = gff.dir)
-        gff <- vroom(gff.dir, delim = "\t", skip = 3, col_names = F)
-        # Add other tracks
-        # variants_track <- track_variant()
-        # alignments_track <- track_alignments()
-    } else if(is.null(loadJBrowse_fasta())){
-      stop("Upload the genome information in upload session to access this feature.")
-    }
+    validate(
+      need(is.null(loadJBrowse_fasta()) & !is.null(loadExample()), "Upload the genome information in upload session to access this feature.")
+    )
+    
+    path.fa <- loadExample()$fasta
+    path.gff <- loadExample()$gff3
+    
+    ext.list <- strsplit(c(loadExample()$fasta,loadExample()$gff3), "[.]")
+    
+    ext <- sapply(ext.list, function(x) {
+      if(x[length(x)] == "gz") paste0(x[length(x)-1], ".",x[length(x)])
+    })
+    
+    # fasta.dir <- paste0(tempfile(),".", ext[1])
+    # download.file(loadExample()$fasta, destfile = fasta.dir)
+    # download.file(paste0(loadExample()$fasta, ".fai"), destfile = paste0(fasta.dir, ".fai"))
+    # path.fa <- fasta.dir
+    
+    gff.dir <- paste0(tempfile(),".", ext[2])
+    download.file(loadExample()$gff3, destfile = gff.dir)
+    #path.gff <- gff.dir
+    
+    gff <- vroom(gff.dir, delim = "\t", skip = 3, col_names = F, progress = FALSE, show_col_types = FALSE)
+    # Add other tracks
+    # variants_track <- track_variant()
+    # alignments_track <- track_alignments()
     
     if(!grepl("^http", path.fa)){
       data_server <- serve_data(dirname(path.fa), port = 5000)
@@ -372,12 +457,12 @@ mod_genes_view_server <- function(input, output, session,
   
   # Link the UI with the browser widget
   output$browserOutput <- renderJBrowseR({
-    if(reset()) stop("The server is off, you can now submit new files in the upload tab.")
+    if(reset()) stop(safeError("The server is off, you can now submit new files in the upload tab."))
     
-    if(!is.null(loadMap()))
-      if(loadMap()$software == "polymapR") stop("Feature not implemented for software polymapR.")
-    
-    if(is.null(loadMap()$ph.p1)) stop("Upload map information in the upload session to access this feature.")
+    validate(
+      need(!(!is.null(loadMap()) & loadMap()$software == "polymapR"), "Feature not implemented for software polymapR."),
+      need(!is.null(loadMap()$ph.p1), "Upload map information in the upload session to access this feature.") 
+    )
     
     if(!grepl("^http", button()$path.fa)){
       assembly <- assembly(
@@ -459,7 +544,9 @@ mod_genes_view_server <- function(input, output, session,
     mks.range.1 <- mks$g.dist[mks.range[1]]
     mks.range.2 <- mks$g.dist[mks.range[length(mks.range)]]
     
-    if(mks.range.1 > mks.range.2) stop("Inverted region. Check graphic `Genomic position (bp) x Linkage Map position (cM)`")
+    validate(
+      need(mks.range.1 < mks.range.2, "Inverted region. Check graphic `Genomic position (bp) x Linkage Map position (cM)`")
+    )
     
     tracks_set <- c(annotations_track, vcf_track, align_track, wiggle_track)
     
@@ -488,29 +575,27 @@ mod_genes_view_server <- function(input, output, session,
   })
   
   output$genes_ano  <- DT::renderDataTable(server = FALSE, {
-    if(!is.null(loadMap()))
-      if(loadMap()$software == "polymapR") stop("Feature not implemented for software polymapR.")    
+    validate(
+      need(!(!is.null(loadMap()) & loadMap()$software == "polymapR"), "Feature not implemented for software polymapR."),
+      need(!is.null(loadMap()$ph.p1), "Upload map information in the upload session to access this feature."),
+      need(!is.null(button()$gff), "Upload annotation file (.gff3) in the upload session to access this feature.")
+    )
     
-    if(is.null(loadMap()$ph.p1)) stop("Upload map information in the upload session to access this feature.")
-    
-    if(!is.null(button()$gff)) {
-      group <- as.numeric(input$group)
-      mks<- loadMap()$maps[[group]]
-      mks <- mks[order(mks$l.dist),]
-      mks.range <- which(mks$l.dist >= input$range[1] &  mks$l.dist <= input$range[2])
-      mks.range.1 <- mks$g.dist[mks.range[1]]
-      mks.range.2 <- mks$g.dist[mks.range[length(mks.range)]]
-      df <- button()$gff
-      colnames(df) <- c("seqid", "source", "type", "start", "end", "score", "strand", "phase", "attributes")
-      df <- df %>% filter(seqid == unique(mks$g.chr) & start > mks.range.1 & end < mks.range.2)
-      DT::datatable(df, extensions = 'Buttons',
-                    options = list(
-                      dom = 'Bfrtlp',
-                      buttons = c('copy', 'csv', 'excel', 'pdf')
-                    ),
-                    class = "display")
-    } else 
-      stop("Upload annotation file (.gff3) in the upload session to access this feature.")
+    group <- as.numeric(input$group)
+    mks<- loadMap()$maps[[group]]
+    mks <- mks[order(mks$l.dist),]
+    mks.range <- which(mks$l.dist >= input$range[1] &  mks$l.dist <= input$range[2])
+    mks.range.1 <- mks$g.dist[mks.range[1]]
+    mks.range.2 <- mks$g.dist[mks.range[length(mks.range)]]
+    df <- button()$gff
+    colnames(df) <- c("seqid", "source", "type", "start", "end", "score", "strand", "phase", "attributes")
+    df <- df %>% filter(seqid == unique(mks$g.chr) & start > mks.range.1 & end < mks.range.2)
+    DT::datatable(df, extensions = 'Buttons',
+                  options = list(
+                    dom = 'Bfrtlp',
+                    buttons = c('copy', 'csv', 'excel', 'pdf')
+                  ),
+                  class = "display")
   })
   
   ## Downloads
@@ -519,7 +604,7 @@ mod_genes_view_server <- function(input, output, session,
   fn_downloadname <- reactive({
     seed <- sample(1:1000,1)
     if(input$fformat=="png") filename <- paste0("profile","_",seed,".png")
-    if(input$fformat=="tiff") filename <- paste0("profile","_",seed,".tif")
+    if(input$fformat=="tiff") filename <- paste0("profile","_",seed,".tiff")
     if(input$fformat=="jpeg") filename <- paste0("profile","_",seed,".jpg")
     if(input$fformat=="pdf") filename <- paste0("profile","_",seed,".pdf")
     return(filename)
@@ -537,15 +622,27 @@ mod_genes_view_server <- function(input, output, session,
                        by_range=T, 
                        software = loadQTL()$software)
     ggsave(pl, filename = fn_downloadname(), 
-           width = 12.7, height = 8, units = "in")    
+           width = input$width_profile, height = input$height_profile, 
+           units = "mm", dpi = input$dpi_profile)    
   }
   
+  observe({
+    if (!is.null(loadQTL()) & input$width_profile > 1 & input$height_profile > 1 & input$dpi_profile > 1) {
+      Sys.sleep(1)
+      # enable the download button
+      shinyjs::enable("bn_download")
+    } else {
+      shinyjs::disable("bn_download")
+    }
+  })
+  
   # download handler
   output$bn_download <- downloadHandler(
     filename = fn_downloadname,
     content = function(file) {
       fn_download()
       file.copy(fn_downloadname(), file, overwrite=T)
+      file.remove(fn_downloadname())
     }
   )
   
@@ -553,7 +650,7 @@ mod_genes_view_server <- function(input, output, session,
   fn_downloadname_phi <- reactive({
     seed <- sample(1:1000,1)
     if(input$fformat_phi=="png") filename <- paste0("linkageXphisical","_",seed,".png")
-    if(input$fformat_phi=="tiff") filename <- paste0("linkageXphisical","_",seed,".tif")
+    if(input$fformat_phi=="tiff") filename <- paste0("linkageXphisical","_",seed,".tiff")
     if(input$fformat_phi=="jpeg") filename <- paste0("linkageXphisical","_",seed,".jpg")
     if(input$fformat_phi=="pdf") filename <- paste0("linkageXphisical","_",seed,".pdf")
     return(filename)
@@ -578,15 +675,29 @@ mod_genes_view_server <- function(input, output, session,
       theme_bw() + theme(legend.position = "none") 
     
     ggsave(p, filename = fn_downloadname_phi(), 
-           width = 12.7, height = 8, units = "in")    
+           width = input$width_phi, height = input$height_phi, 
+           units = "mm", dpi = input$dpi_phi)    
   }
   
+  observe({
+    if (!is.null(loadMap()) & input$width_phi > 1 & input$height_phi > 1 & input$dpi_phi > 1) {
+      if (loadMap()$software != "polymapR" ) {
+        Sys.sleep(1)
+        # enable the download button
+        shinyjs::enable("bn_download_phi")
+      } else {
+        shinyjs::disable("bn_download_phi")
+      }
+    } else shinyjs::disable("bn_download_phi")
+  })
+  
   # download handler
   output$bn_download_phi <- downloadHandler(
     filename = fn_downloadname_phi,
     content = function(file) {
       fn_download_phi()
       file.copy(fn_downloadname_phi(), file, overwrite=T)
+      file.remove(fn_downloadname_phi())
     }
   )
   
