@@ -4,7 +4,7 @@
 #'
 #' @param id,input,output,session Internal parameters for {shiny}.
 #'
-#' @importFrom shinyjs inlineCSS
+#' @importFrom shinyjs inlineCSS useShinyjs
 #' @importFrom plotly plotlyOutput
 #' @importFrom shiny NS tagList  
 #' 
@@ -24,14 +24,21 @@ mod_map_view_ui <- function(id){
           ),
           column(width = 12,
                  div(style = "position:absolute;right:1em;", 
-                     actionButton(ns("exit"), "Exit",icon("times-circle"), class = "btn btn-danger"),
+                     div(
+                       actionButton(ns("goGenes"), "Go to Genome",icon("arrow-circle-left", verify_fa = FALSE), class = "btn btn-primary"))
                  )
           ),
           tags$h2(tags$b("VIEWmap")), br(), hr(),
           
           column(6,
+                 column(12,
+                        box(
+                          background = "light-blue",
+                          "Required inputs (*)", br(),
+                        )
+                 ),
                  column(6,
-                        box(width = 12, solidHeader = TRUE,  status="info", title = h4("Select phenotypes"),
+                        box(width = 12, solidHeader = TRUE,  status="info", title = "Select phenotypes *",
                             pickerInput(ns("phenotypes"),
                                         label = h4("Phenotypes:"),
                                         choices = "This will be updated",
@@ -45,7 +52,7 @@ mod_map_view_ui <- function(id){
                         ), br(),
                  ),
                  column(6,
-                        box(width = 12, solidHeader = TRUE, status="info", title = h4("Select linkage group"),
+                        box(width = 12, solidHeader = TRUE, status="info", title = "Select linkage group *",
                             selectInput(inputId = ns("group"), label = p("Linkage group:"), choices = 1:15, selected = 1),
                             checkboxInput(ns("op"), label = "Show SNP names", value = TRUE)
                         ), br(),
@@ -57,44 +64,91 @@ mod_map_view_ui <- function(id){
                       value = c(0, 20), step = 1), 
           uiOutput(ns("interval"))
         ),
-        box(width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = TRUE, status="primary", title = h4("QTL profile"),
-            column(2,
-                   downloadBttn(ns('bn_download'), style = "gradient", color = "royal")
+        box(id= ns("box_profile"), width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = TRUE, status="primary", title = actionLink(inputId = ns("profileID"), label = "QTL profile"),
+            column(12,
+                   box(
+                     width = 5, background = "light-blue",
+                     "* QTL analysis files or viewpoly object or example dataset (check `Input data` tab)", 
+                   )
+            ), 
+            column(3,
+                   tags$head(tags$style(".butt{background-color:#add8e6; border-color: #add8e6; color: #337ab7;}")),
+                   useShinyjs(),
+                   downloadButton(ns('bn_download'), "Download", class = "butt")
             ),
-            column(10,
+            column(3,
                    radioButtons(ns("fformat"), "File type", choices=c("png","tiff","jpeg","pdf"), selected = "png", inline = T)
+            ),                     
+            column(2,
+                   numericInput(ns("width_profile"), "Width (mm)", value = 180),
+            ),
+            column(2,
+                   numericInput(ns("height_profile"), "Height (mm)", value = 120),
+            ),
+            column(2,
+                   numericInput(ns("dpi_profile"), "DPI", value = 300)
             ), br(),
             column(12,
                    hr(),
                    plotlyOutput(ns("plot_qtl")), 
             )
         ), br(),
-        box(width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = h4("Map"),
-            column(2,
-                   downloadBttn(ns('bn_download_map'), style = "gradient", color = "royal")
+        box(id = ns("box_map"), width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = actionLink(inputId = ns("mapID"), label = "Map"),
+            column(12,
+                   box(
+                     width = 5, background = "light-blue",
+                     "* Linkage map files or viewpoly object or example dataset (check `Input data` tab)", 
+                   )
+            ), 
+            column(3,
+                   downloadButton(ns('bn_download_map'), "Download", class = "butt")
             ),
-            column(10,
+            column(3,
                    radioButtons(ns("fformat_map"), "File type", choices=c("png","tiff","jpeg","pdf"), selected = "png", inline = T)
+            ),                     
+            column(2,
+                   numericInput(ns("width_map"), "Width (mm)", value = 180),
+            ),
+            column(2,
+                   numericInput(ns("height_map"), "Height (mm)", value = 120),
+            ),
+            column(2,
+                   numericInput(ns("dpi_map"), "DPI", value = 300)
             ), br(),
             column(12,
                    hr(),
                    plotOutput(ns("plot_map"), height = "500px"), br(),
-                   includeHTML(system.file("ext", "include.html", package = "viewpoly")), br(), br(),
-                   box(width = 12, solidHeader = FALSE, collapsible = TRUE,  collapsed = TRUE, status="primary", title = h4("Parents haplotypes table"),
+                   includeHTML(system.file(package = "viewpoly", "ext/include.html")), br(), br(),
+                   box(id = ns("box_phaplo"),width = 12, solidHeader = FALSE, collapsible = TRUE,  collapsed = TRUE, status="primary", title = actionLink(inputId = ns("phaploID"), label = "Parents haplotypes table"),
                        DT::dataTableOutput(ns("parents_haplo"))
                    )
             )
         ),
-        box(width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = TRUE, status="primary", title = h4("Map summary"),
+        box(id = ns("box_mapsumm"), width = 12, solidHeader = TRUE, collapsible = TRUE,  collapsed = FALSE, status="primary", title = actionLink(inputId = ns("mapsummID"), label = "Map summary"),
+            column(12,
+                   box(
+                     width = 5, background = "light-blue",
+                     "* Linkage map files or viewpoly object or example dataset (check `Input data` tab)", 
+                   )
+            ), 
             column(12,
                    DT::dataTableOutput(ns("summary")), br(), hr()
             ),
-            column(2,
-                   downloadBttn(ns('bn_download_summary'), style = "gradient", color = "royal")
+            column(3,
+                   downloadButton(ns('bn_download_summary'), "Download", class = "butt")
             ),
-            column(10,
+            column(3,
                    radioButtons(ns("fformat_summary"), "File type", choices=c("png","tiff","jpeg","pdf"), selected = "png", inline = T), br(),
-            ), 
+            ),                     
+            column(2,
+                   numericInput(ns("width_summary"), "Width (mm)", value = 180),
+            ),
+            column(2,
+                   numericInput(ns("height_summary"), "Height (mm)", value = 120),
+            ),
+            column(2,
+                   numericInput(ns("dpi_summary"), "DPI", value = 300)
+            ), br(),
             column(12,
                    hr(),
                    plotOutput(ns("map_summary"))
@@ -109,7 +163,7 @@ mod_map_view_ui <- function(id){
 #'
 #' @importFrom plotly ggplotly renderPlotly
 #' @importFrom dplyr `%>%`
-#'
+#' @importFrom shinyjs js
 #' @noRd 
 mod_map_view_server <- function(input, output, session, 
                                 loadMap, loadQTL,
@@ -118,8 +172,26 @@ mod_map_view_server <- function(input, output, session,
   
   pheno <- LG <- NULL
   
-  observeEvent(input$exit, {
-    stopApp()
+  #Collapse boxes
+  observeEvent(input$profileID, {
+    js$collapse(ns("box_profile"))
+  })
+  
+  observeEvent(input$mapID, {
+    js$collapse(ns("box_map"))
+  })
+  
+  observeEvent(input$mapsummID, {
+    js$collapse(ns("box_mapsumm"))
+  })
+  
+  observeEvent(input$phaploID, {
+    js$collapse(ns("box_phaplo"))
+  })
+  
+  observeEvent(input$goGenes, {
+    updateTabsetPanel(session = parent_session, inputId = "viewpoly",
+                      selected = "genes")
   })
   
   observe({
@@ -234,95 +306,101 @@ mod_map_view_server <- function(input, output, session,
   
   # Plot QTL profile
   output$plot_qtl <- renderPlotly({
-    if(!is.null(loadQTL())){
-      idx <- which(unique(loadQTL()$profile$pheno) %in% input$phenotypes)
-      pl <- plot_profile(profile = loadQTL()$profile, 
-                         qtl_info = loadQTL()$qtl_info, 
-                         selected_mks = loadQTL()$selected_mks,
-                         pheno.col = idx,
-                         lgs.id = as.numeric(input$group),
-                         range.min = input$range[1],
-                         range.max = input$range[2], by_range=T)
-      
-      ggplotly(source = "qtl_profile", pl, tooltip=c("Trait", "Position (cM)")) %>% layout(legend = list(orientation = 'h', y = -0.3))
-    } else 
-      stop("Upload the QTL information in upload session to access this feature.")
+    validate(
+      need(!is.null(loadQTL()), "Upload the QTL information in upload session to access this feature.")
+    )
+    idx <- which(unique(loadQTL()$profile$pheno) %in% input$phenotypes)
+    pl <- plot_profile(profile = loadQTL()$profile, 
+                       qtl_info = loadQTL()$qtl_info, 
+                       selected_mks = loadQTL()$selected_mks,
+                       pheno.col = idx,
+                       lgs.id = as.numeric(input$group),
+                       range.min = input$range[1],
+                       range.max = input$range[2], by_range=T)
+    
+    ggplotly(source = "qtl_profile", pl, tooltip=c("Trait", "Position (cM)")) %>% 
+      layout(legend = list(orientation = 'h', y = -0.3), 
+             modebar = list(
+               remove = c("toImage", 
+                          "hovercompare", 
+                          "hoverCompareCartesian")),
+             clickmode ="none",
+             dragmode = FALSE)
   })
   
   # Plot map
   output$plot_map <- renderPlot({
-    if(!is.null(loadMap()$ph.p1)) {
-      maps <- lapply(loadMap()$maps, function(x) {
-        y <- x$l.dist
-        names(y) <- x$mk.names
-        y
-      })
-      draw_map_shiny(left.lim = input$range[1], 
-                     right.lim = input$range[2], 
-                     ch = input$group,
-                     d.p1 = loadMap()$d.p1,
-                     d.p2 = loadMap()$d.p2, 
-                     maps = maps, 
-                     ph.p1 = loadMap()$ph.p1, 
-                     ph.p2 = loadMap()$ph.p2,
-                     snp.names = input$op)
-      
-      max_updated = reactive({
-        map_summary(left.lim = input$range[1], right.lim = input$range[2], 
-                    ch = input$group, maps = maps, 
-                    d.p1 = loadMap()$d.p1, 
-                    d.p2 = loadMap()$d.p2)[[5]]
-      })
-      
-      observeEvent(max_updated, {
-        updateSliderInput(inputId = "range", max = round(max_updated(),2))
-      })
-    } else 
-      stop("Upload map information in the upload session to access this feature.")
+    validate(
+      need(!is.null(loadMap()$ph.p1), "Upload map information in the upload session to access this feature.")
+    )
+    maps <- lapply(loadMap()$maps, function(x) {
+      y <- x$l.dist
+      names(y) <- x$mk.names
+      y
+    })
+    draw_map_shiny(left.lim = input$range[1], 
+                   right.lim = input$range[2], 
+                   ch = input$group,
+                   d.p1 = loadMap()$d.p1,
+                   d.p2 = loadMap()$d.p2, 
+                   maps = maps, 
+                   ph.p1 = loadMap()$ph.p1, 
+                   ph.p2 = loadMap()$ph.p2,
+                   snp.names = input$op)
+    
+    max_updated = reactive({
+      map_summary(left.lim = input$range[1], right.lim = input$range[2], 
+                  ch = input$group, maps = maps, 
+                  d.p1 = loadMap()$d.p1, 
+                  d.p2 = loadMap()$d.p2)[[5]]
+    })
+    
+    observeEvent(max_updated, {
+      updateSliderInput(inputId = "range", max = round(max_updated(),2))
+    })
   })
   
   output$parents_haplo  <- DT::renderDataTable(server = FALSE, {
-    if(!is.null(loadMap()$ph.p1)) {
-      group <- as.numeric(input$group)
-      mks<- loadMap()$maps[[group]]
-      mks <- mks[order(mks$l.dist),]
-      mks.range <- which(mks$l.dist >= input$range[1] &  mks$l.dist <= input$range[2])
-      p1 <- loadMap()$ph.p1[[group]][mks.range,]
-      #colnames(p1) <- paste0("p1.",1:dim(p1)[2])
-      p2 <- loadMap()$ph.p2[[group]][mks.range,]
-      #colnames(p2) <- paste0("p2.",1:dim(p2)[2])
-      p.haplo <- cbind(p1,p2)
-      
-      DT::datatable(p.haplo, extensions = 'Buttons', 
-                options = list(
-                  dom = 'Bfrtlp',
-                  buttons = c('copy', 'csv', 'excel', 'pdf')
-                ),
-                class = "display")
-    } else 
-      stop("Upload map information in the upload session to access this feature.")
+    validate(
+      need(!is.null(loadMap()$ph.p1), "Upload map information in the upload session to access this feature.")
+    )
+    group <- as.numeric(input$group)
+    mks<- loadMap()$maps[[group]]
+    mks <- mks[order(mks$l.dist),]
+    mks.range <- which(mks$l.dist >= input$range[1] &  mks$l.dist <= input$range[2])
+    p1 <- loadMap()$ph.p1[[group]][mks.range,]
+    #colnames(p1) <- paste0("p1.",1:dim(p1)[2])
+    p2 <- loadMap()$ph.p2[[group]][mks.range,]
+    #colnames(p2) <- paste0("p2.",1:dim(p2)[2])
+    p.haplo <- cbind(p1,p2)
+    
+    DT::datatable(p.haplo, extensions = 'Buttons', 
+                  options = list(
+                    dom = 'Bfrtlp',
+                    buttons = c('copy', 'csv', 'excel', 'pdf')
+                  ),
+                  class = "display")
   })
   
   # Map summary
   output$summary  <- DT::renderDataTable(server = FALSE, {
-    if(!is.null(loadMap()$ph.p1)) {
-      summary <- summary_maps(loadMap())
-      
-      DT::datatable(summary, extensions = 'Buttons', 
-                options = list(
-                  dom = 'Bfrtlp',
-                  buttons = c('copy', 'csv', 'excel', 'pdf')
-                ),
-                class = "display")
-    } else 
-      stop("Upload map information in the upload session to access this feature.")
+    validate(
+      need(!is.null(loadMap()$ph.p1), "Upload map information in the upload session to access this feature.")
+    )
+    summary <- summary_maps(loadMap())
+    DT::datatable(summary, extensions = 'Buttons', 
+                  options = list(
+                    dom = 'Bfrtlp',
+                    buttons = c('copy', 'csv', 'excel', 'pdf') 
+                  ),
+                  class = "display")
   })
   
   output$map_summary <- renderPlot({
-    if(!is.null(loadMap()$ph.p1)) {
-      plot_map_list(loadMap())
-    } else 
-      stop("Upload map information in the upload session to access this feature.")
+    validate(
+      need(!is.null(loadMap()$ph.p1), "Upload map information in the upload session to access this feature.")
+    )
+    plot_map_list(loadMap())
   })
   
   ### Downloads
@@ -331,7 +409,7 @@ mod_map_view_server <- function(input, output, session,
   fn_downloadname <- reactive({
     seed <- sample(1:1000,1)
     if(input$fformat=="png") filename <- paste0("profile","_",seed,".png")
-    if(input$fformat=="tiff") filename <- paste0("profile","_",seed,".tif")
+    if(input$fformat=="tiff") filename <- paste0("profile","_",seed,".tiff")
     if(input$fformat=="jpeg") filename <- paste0("profile","_",seed,".jpg")
     if(input$fformat=="pdf") filename <- paste0("profile","_",seed,".pdf")
     return(filename)
@@ -349,15 +427,27 @@ mod_map_view_server <- function(input, output, session,
                        by_range=T, 
                        software = loadQTL()$software)
     ggsave(pl, filename = fn_downloadname(), 
-           width = 12.7, height = 8, units = "in")    
+           width = input$width_profile, height = input$height_profile, 
+           units = "mm", dpi = input$dpi_profile)    
   }
   
+  observe({
+    if (!is.null(loadQTL()) & input$width_profile > 1 & input$height_profile > 1 & input$dpi_profile > 1) {
+      Sys.sleep(1)
+      # enable the download button
+      shinyjs::enable("bn_download")
+    } else {
+      shinyjs::disable("bn_download")
+    }
+  })
+  
   # download handler
   output$bn_download <- downloadHandler(
     filename = fn_downloadname,
     content = function(file) {
       fn_download()
       file.copy(fn_downloadname(), file, overwrite=T)
+      file.remove(fn_downloadname())
     }
   )
   
@@ -366,7 +456,7 @@ mod_map_view_server <- function(input, output, session,
     png <- tiff <- jpeg <- pdf <- dev.off <- NULL
     seed <- sample(1:1000,1)
     if(input$fformat_map=="png") filename <- paste0("map","_",seed,".png")
-    if(input$fformat_map=="tiff") filename <- paste0("map","_",seed,".tif")
+    if(input$fformat_map=="tiff") filename <- paste0("map","_",seed,".tiff")
     if(input$fformat_map=="jpeg") filename <- paste0("map","_",seed,".jpg")
     if(input$fformat_map=="pdf") filename <- paste0("map","_",seed,".pdf")
     return(filename)
@@ -382,13 +472,13 @@ mod_map_view_server <- function(input, output, session,
     })
     
     if(input$fformat_map == "png"){
-      png(fn_downloadname_map(), width = 22, height = 5, units = "in", res = 300)
+      png(fn_downloadname_map(), width = input$width_map, height = input$height_map, units = "mm", res = input$dpi_map)
     } else  if(input$fformat_map == "tiff"){
-      tiff(fn_downloadname_map(), width = 22, height = 5, units = "in", res = 300)
+      tiff(fn_downloadname_map(), width = input$width_map, height = input$height_map, units = "mm", res = input$dpi_map)
     } else  if(input$fformat_map == "jpeg"){
-      jpeg(fn_downloadname_map(), width = 22, height = 5, units = "in", res = 300)
+      jpeg(fn_downloadname_map(), width = input$width_map, height = input$height_map, units = "mm", res = input$dpi_map)
     } else  if(input$fformat_map == "pdf"){
-      pdf(fn_downloadname_map(), width = 22, height = 5, units = "in", res = 300)
+      pdf(fn_downloadname_map(), width = input$width_map, height = input$height_map, units = "mm", res = input$dpi_map)
     }
     
     draw_map_shiny(left.lim = input$range[1], 
@@ -404,12 +494,23 @@ mod_map_view_server <- function(input, output, session,
     dev.off()
   }
   
+  observe({
+    if (!is.null(loadMap()) & input$width_map > 1 & input$height_map > 1 & input$dpi_map > 1) {
+      Sys.sleep(1)
+      # enable the download button
+      shinyjs::enable("bn_download_map")
+    } else {
+      shinyjs::disable("bn_download_map")
+    }
+  })
+  
   # download handler
   output$bn_download_map <- downloadHandler(
     filename = fn_downloadname_map,
     content = function(file) {
       fn_download_map()
       file.copy(fn_downloadname_map(), file, overwrite=T)
+      file.remove(fn_downloadname_map())
     }
   )
   
@@ -418,7 +519,7 @@ mod_map_view_server <- function(input, output, session,
     png <- tiff <- jpeg <- pdf <- dev.off <- NULL
     seed <- sample(1:1000,1)
     if(input$fformat_summary=="png") filename <- paste0("map","_",seed,".png")
-    if(input$fformat_summary=="tiff") filename <- paste0("map","_",seed,".tif")
+    if(input$fformat_summary=="tiff") filename <- paste0("map","_",seed,".tiff")
     if(input$fformat_summary=="jpeg") filename <- paste0("map","_",seed,".jpg")
     if(input$fformat_summary=="pdf") filename <- paste0("map","_",seed,".pdf")
     return(filename)
@@ -436,13 +537,13 @@ mod_map_view_server <- function(input, output, session,
     })
     
     if(input$fformat_summary == "png"){
-      png(fn_downloadname_summary(),  width = 12.7, height = 8, units = "in", res = 300)
+      png(fn_downloadname_summary(),  width = input$width_summary, height = input$height_summary, units = "mm", res = input$dpi_summary)
     } else  if(input$fformat_summary == "tiff"){
-      tiff(fn_downloadname_summary(),  width = 12.7, height = 8, units = "in", res = 300)
+      tiff(fn_downloadname_summary(),  width = input$width_summary, height = input$height_summary, units = "mm", res = input$dpi_summary)
     } else  if(input$fformat_summary == "jpeg"){
-      jpeg(fn_downloadname_summary(),  width = 12.7, height = 8, units = "in", res = 300)
+      jpeg(fn_downloadname_summary(),  width = input$width_summary, height = input$height_summary, units = "mm", res = input$dpi_summary)
     } else  if(input$fformat_summary == "pdf"){
-      pdf(fn_downloadname_summary(),  width = 12.7, height = 8, units = "in", res = 300)
+      pdf(fn_downloadname_summary(),  width = input$width_summary, height = input$height_summary, units = "mm", res = input$dpi_summary)
     }
     
     plot_map_list(loadMap())   
@@ -450,12 +551,23 @@ mod_map_view_server <- function(input, output, session,
     dev.off()
   }
   
+  observe({
+    if (!is.null(loadMap()) & input$width_summary > 1 & input$height_summary > 1 & input$dpi_summary > 1) {
+      Sys.sleep(1)
+      # enable the download button
+      shinyjs::enable("bn_download_summary")
+    } else {
+      shinyjs::disable("bn_download_summary")
+    }
+  })
+  
   # download handler
   output$bn_download_summary <- downloadHandler(
     filename = fn_downloadname_summary,
     content = function(file) {
       fn_download_summary()
       file.copy(fn_downloadname_summary(), file, overwrite=T)
+      file.remove(fn_downloadname_summary())
     }
   )
 }
